package org.mdp.cli;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Map;

import org.mdp.dir.User;
import org.mdp.dir.UserDirectoryStub;
import org.mdp.im.InstantMessagingServer;


public class InstantMessagingApp {

	static String LOGO = 
			"========================================\n"+
					" _ __ ___   ___ _ __  ___  __ _ (_) ___\n"+ 
					"| '_ ` _ \\ / _ \\ '_ \\/ __|/ _` || |/ _ \n"+
					"| | | | | |  __/ | | \\__ \\ (_| || |  __/\n"+
					"|_| |_| |_|\\___|_| |_|___/\\__,_|/ |\\___|\n"+
					"                               |__/     \n"+
					"========================================\n";

	static String INTRO = LOGO +"... a revolution in communication!\n"
			+ "* Type 'list' for up-to-date directory, 'add' to add yourself to the directory, 'rmv' to remove yourself from directory.\n"
			+ "* Otherwise type the username (in the directory) of whom you want to message.";


	/** 
	 * @param args
	 * @throws IOException
	 * @throws AlreadyBoundException
	 * @throws NotBoundException 
	 */
	public static void main(String args[]) throws IOException, AlreadyBoundException, NotBoundException{
		System.out.println(INTRO);

		// Create a user for yourself
		// Please don't call your username: 'list' 'rmv' or 'add' :P
		// If you like, pass info through the command-line instead
		// of hard-coding (optional)
		User me = new User("test", "Aidan Hogan", "localhost", 1915);

		// Then start your RMI registry and your messaging server to
		// receive messages
		startMessagingServer(me.getPort());

		// Connect to the directory
		UserDirectoryStub uds = connectToDirectory();

		Map<String,User> dir = uds.getDirectory();

		while(true){
			System.out.println("\n\nWhich user would you like to message ...");

			BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
			String line = br.readLine().trim();

			if(line.equals("list")){
				dir = uds.getDirectory();
				System.err.println("\n\nDirectory of usernames ...");
				for(String s: dir.keySet()){
					System.err.println(s+" ");
				}
			} else if(line.equals("rmv")){
				uds.removeUserWithName(me.getUsername());
			} else if(line.equals("add")){
				uds.createUser(me);
			} else{
				String username = line;
				User user = dir.get(username);
				if(user==null){
					System.out.println("\n\nCould not find the user "+username+" in the directory. (Type 'list' to refresh directory.)");
				} else{
					System.out.println("\n\nEnter your message ...");
					
					line = br.readLine().trim();
	
					try{
						long ms = messageUser(user,me,line);
						System.out.println("\n\nMessage received by "+username+" at time "+ms);
					} catch(Exception e){
						System.out.println("\n\nCould not message user "+username+": "+e.getMessage());
					}
				}
			}
		}

	}

	/**
	 * I'm very generous and I've done this for you.
	 * 
	 * Starts your messaging server to listen on the given port.
	 * 
	 * @param port
	 * @return
	 * @throws RemoteException
	 * @throws AlreadyBoundException
	 */
	public static Registry startMessagingServer(int port) throws RemoteException, AlreadyBoundException {
		// open the registry on the port
		Registry r = LocateRegistry.createRegistry(port);

		// create a stub for the instant messaging server
		Remote stub = UnicastRemoteObject.exportObject(new InstantMessagingServer(),0);

		// just use the class name for the stub (any name would be fine
		// but client needs to know the name to find the stub)
		String stubname = InstantMessagingServer.class.getSimpleName();

		// bind the stub to the registry under the given name
		r.bind(stubname, stub);

		// return the registry
		return r;
	}

	/**
	 * Connect to the central directory.
	 * 
	 * (See UserDirectoryClient for example)
	 * 
	 * 
	 * @throws RemoteException 
	 * @throws NotBoundException 
	 */
	public static UserDirectoryStub connectToDirectory() throws RemoteException, NotBoundException {
		
	}

	/**
	 * Message user.
	 * 
	 * Connect to their registry, retrieve the InstantMessagingStub stub.
	 * Call the message method on the stub.
	 * 
	 * @param username
	 * @param msg
	 * @return Time acknowledged by message server
	 * @throws NotBoundException 
	 * @throws RemoteException 
	 * @throws AccessException 
	 */
	public static long messageUser(User to, User from, String msg) throws AccessException, RemoteException, NotBoundException{
		
	}
}
